// Ajoy Raman,www.ajoyraman.in Sept 2019
// This program sweeps angle +/-60 deg and outputs the parameters 
// obtained from two sensors for plotting the two-beams sum and difference
// Added Simple Tracking Demo

#include <Servo.h>
 
#define SERVO_PWM_PIN 9
#define signalOnePin 13
#define testOnePin 12
#define signalTwoPin 11
#define testTwoPin 10
#define adcTimeCheckPin 8

#define ANGLE_BOUNDS 60 //80
#define ANGLE_STEP 1
 
int angle = 0;
int dir = 1;
int i,j;
uint8_t Raw_Data_Buffer[512];
int sensorValue, upper8bits;
uint16_t data_buffer_size=128 ;//max 512 (128 covers 4msec 60cm)
uint16_t dataSumOne,dataSumTwo,sumSignal;
int errorSignal, absErrorSignal,duration,distance;


Servo myservo;

void setup() {
Serial.begin(38400);
analogReference(EXTERNAL);
myservo.attach(SERVO_PWM_PIN);//later will be attached/detached, but once should start for first time
pinMode(signalOnePin, OUTPUT);
pinMode(testOnePin, OUTPUT);
pinMode(signalTwoPin, OUTPUT);
pinMode(testTwoPin, OUTPUT);
pinMode(adcTimeCheckPin, OUTPUT);

//For fast ADC
	// For changing pre-scale 128 to 16  
	// defines for setting and clearing register bits
	#ifndef cbi
	#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
	#endif
	#ifndef sbi
	#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
	#endif
	      // 1 0 0 16 Set~18us
        // 1 0 1 32    ~36us  Set
        // 1 1 0 64    ~75us
        // 1 1 1 128   ~150us
	sbi(ADCSRA,ADPS2) ;
	cbi(ADCSRA,ADPS1) ;
	sbi(ADCSRA,ADPS0) ;
}
 
void loop() { 
	//-------------------------------getting the echo values	
  myservo.detach(); //very important! this command disable PWM - which make interference during analog read
  delay(20); //wait additionally 2 mS for RF noise to settle down (first few centimeters)

  // First Sensor
  //trigger the ultrasonic sensor One & create test signal
  //signalOnePin and testOnePin Low for 2us
  pinMode(signalOnePin, OUTPUT);
  digitalWrite(signalOnePin, LOW);
  digitalWrite(testOnePin, LOW);
  delayMicroseconds(2);
  //signalOnePin and testOnePin High 
  digitalWrite(signalOnePin, HIGH);
  digitalWrite(testOnePin, HIGH);
  delayMicroseconds(5);
  //signalPin low after 5us and change to Input
  digitalWrite(signalOnePin,LOW);
  pinMode(signalOnePin,INPUT);
  //testPin Low after 400us
  delayMicroseconds(400);
  digitalWrite(testOnePin, LOW);

  dataSumOne=0;
   digitalWrite(adcTimeCheckPin, HIGH);
  //read A0 into Data_Buffer  
  for (i = 0; i <data_buffer_size; i++) {
    sensorValue = analogRead(A0);
    upper8bits = (sensorValue>>2) & 0xFF ;
    Raw_Data_Buffer[i]=upper8bits;
    dataSumOne=dataSumOne + upper8bits;
  }
  digitalWrite(adcTimeCheckPin, LOW);
  delay(10); // so that echoes from first Tx do not come into second Rx
  
  // Second Sensor
  //trigger the ultrasonic sensor Two & create test signal
  //signalTwoPin and testTwoPin Low for 2us
  pinMode(signalTwoPin, OUTPUT);
  digitalWrite(signalTwoPin, LOW);
  digitalWrite(testTwoPin, LOW);
  delayMicroseconds(2);
  //signalOnePin and testTwoPin High 
  digitalWrite(signalTwoPin, HIGH);
  digitalWrite(testTwoPin, HIGH);
  delayMicroseconds(5);
  //signaTwoPin low after 5us and change to Input
  digitalWrite(signalTwoPin,LOW);
  pinMode(signalTwoPin,INPUT);
  //testPin Low after 400us
  delayMicroseconds(400);
  digitalWrite(testTwoPin, LOW);

  dataSumTwo=0;
    
  //read A1 into Data_Buffer  
  for (i = 0; i <data_buffer_size; i++) {
    sensorValue = analogRead(A1);
    upper8bits = (sensorValue>>2) & 0xFF ;
    Raw_Data_Buffer[i]=upper8bits;
    dataSumTwo=dataSumTwo + upper8bits;
  }
  
  //----------------print the echo values 
  //dataSumOne = dataSumOne * 1.28;
  dataSumTwo = dataSumTwo * 1.55;
  sumSignal= dataSumOne + dataSumTwo;
  //errorSignal= dataSumOne - dataSumTwo;
  errorSignal= dataSumTwo - dataSumOne;
  absErrorSignal = abs(errorSignal);

  if (absErrorSignal < 150){

      pinMode(signalTwoPin, OUTPUT);
      digitalWrite(signalTwoPin,LOW);     
      delayMicroseconds(2);
      digitalWrite(signalTwoPin, HIGH);  
      delayMicroseconds(5);
      digitalWrite(signalTwoPin,LOW);
      pinMode(signalTwoPin,INPUT);
                  
      duration = pulseIn(signalTwoPin,HIGH);
      
      distance = (duration/2) / 29.1;
    
      //distance = ((duration/2) +200)/ 32;
      
      if (distance >= 400 || distance <= 0){
        Serial.println(distance);//("Out of range");
      }
      else {
        Serial.print(distance);
        Serial.print(" cm");
        Serial.write(0x2C);
        Serial.print(duration);
        Serial.println(" us");
        }
  }
   
    Serial.print(-angle);      //"angle" instead "-angle" will reverse servo direction  
    Serial.write(0x2C);
    Serial.print(dataSumOne);
    Serial.write(0x2C);
    Serial.print(dataSumTwo);
    Serial.write(0x2C);
    Serial.print(sumSignal);
    Serial.write(0x2C);
    Serial.print(errorSignal);
    Serial.write(0x2C);
    Serial.print(absErrorSignal);
    Serial.println();

  myservo.attach(SERVO_PWM_PIN); 
  delay(20);
    //angle = 0;
    myservo.write(angle + ANGLE_BOUNDS); //servo should move one step
//    if (angle >= ANGLE_BOUNDS || angle <= -ANGLE_BOUNDS)
//     {
//       dir = -dir;
//     }
//    angle += (dir*ANGLE_STEP);
    
   if (errorSignal >=0){
       angle--;}
   else {angle++;
   } 

   
  
  delay(50); // 50 alow some time for +5V to settle down, because servo can draw some current and make analog read error
             
 }
 
                 
